"""
Module for accessing an Azure Blob Storage container
and accessing three files containing simplified schedule data
for the zeroG Airlines (ZG).

Attributes:
    SCHEDULE
    FLEET
    AIRPORTS
    MY_CONNECTION
    MY_CONTAINER
"""

import os
import logging

from io import BytesIO

import pandas as pd

from azure.storage.blob import BlobClient
from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient

SCHEDULE      = None
FLEET         = None
AIRPORTS      = None
MY_CONNECTION = None
MY_CONTAINER  = None


# logger = logging.getLogger(
#         "azure.core.pipeline.policies.http_logging_policy")
# 
# logger.setLevel(logging.ERROR)

def _retrieve_file(file_name):
    """Function to retrieve data from Azure Blob Storage container.

    Args:
        file_name (string)

    Returns:
        final_data (pandas DataFrame)

    """
    blob = BlobClient.from_connection_string(
            conn_str       = MY_CONNECTION,
            container_name = MY_CONTAINER,
            blob_name      = file_name)

    data = blob.download_blob().readall()

    file_ext = file_name.split(".")[-1]

    final_data = pd.DataFrame([])

    if file_ext == "json":
        final_data = pd.read_json(BytesIO(data))
    elif file_ext == "csv":
        final_data = pd.read_csv(BytesIO(data))
    else:
        raise Exception("Unsupported file extension. Should be '.csv' or \
                '.json'.")

    return final_data

def get_data():
    """Function to retrieve SCHEDULE, FLEET and AIRPORT from Azure
    Blob Storage container
    """
    global MY_CONNECTION, MY_CONTAINER
    global SCHEDULE, FLEET, AIRPORTS

    def get_key(secret_name):
        key_vault_name = os.environ["KEY_VAULT_NAME"]
        key_vault_address = f"https://{key_vault_name}.vault.azure.net"

        credential = DefaultAzureCredential()

        client = SecretClient(
                vault_url=key_vault_address,
                credential=credential)

        return client.get_secret(secret_name).value

    account_name = "AccountName=zerogrecruiting;"
    key = "AccountKey=" + get_key("zeroGAccountKey") + ";"
    endpoints_protocol = "DefaultEndpointsProtocol=https;"
    endpoints_suffix = "EndpointSuffix=core.windows.net"

    MY_CONNECTION = (
            endpoints_protocol +
            account_name +
            key +
            endpoints_suffix)

    MY_CONTAINER = "python-case-study"

    SCHEDULE = _retrieve_file("schedule.json")
    FLEET    = _retrieve_file("fleet.csv")
    AIRPORTS = _retrieve_file("airports.csv")
