"""
Module for retrieving and processing schedule data on the zeroG Airlines.

Attributes:
    SCHEDULE
    FLEET
    AIRPORTS
"""

import json
import argparse
import logging
import yaml

from geopy import distance

from pygments import highlight, lexers, formatters

from .package import data


SCHEDULE = None
FLEET    = None
AIRPORTS = None
LOGGER = logging.getLogger('schedule_data_processing')


def _retrieve_data():
    """Retrieves schedule, fleet and airport data from the data module.
    They are stored in pandas DataFrames as module level variables.
    """
    global SCHEDULE, FLEET, AIRPORTS

    data.get_data()

    SCHEDULE = data.SCHEDULE
    FLEET    = data.FLEET
    AIRPORTS = data.AIRPORTS

def init():
    """Function to initialize logger and to retrieve schedule, fleet and
    airport data.
    """

    # logger config
    file_handler = logging.FileHandler('log.txt')
    LOGGER.addHandler(file_handler) 
    LOGGER.setLevel(logging.INFO)

    _retrieve_data()

def _get_airport_coord(airport_code):
    """Return latitude and longitude coordinates of airport

    Args:
        airport_code (string) : IATA airport code

    Returns
        airport_coord (tuple) : tuple of latitude (float)
                                and longitude (float)

    """

    airport = AIRPORTS[AIRPORTS['Airport'] == airport_code]

    airport_lat = float(airport['Lat'].to_string(index=False))
    airport_lon = float(airport['Lon'].to_string(index=False))

    airport_coord = (airport_lat, airport_lon)

    return airport_coord

def _calculate_nmiles(departure_airport_code, arrival_airport_code):
    """Returns the distance in nautical miles between
    departure and arrival airports.

    Args:
        departure_airport_code (string) : IATA departure airport code
        arrival_airport_code (string)   : IATA arrival airport code

    Returns:
        distance_nm (float) : distance in nautical miles rounded
                              to 2 decimal places

    """

    departure_airport_coord = _get_airport_coord(departure_airport_code)

    arrival_airport_coord   = _get_airport_coord(arrival_airport_code)

    distance_nm = distance.distance(
            departure_airport_coord,
            arrival_airport_coord).nautical

    distance_nm = round(distance_nm, 2)

    return distance_nm

def _format_flight_record(flight_record, pretty):
    """Function to format flight records the value of 'pretty'

    Args:
        flight_record (dict)
        pretty (int) {0, 1, 2, 3} : level of formatting
            0 : json unformatted (default).
            1 : json formatted.
            2 : json formatted & colorful.
            3 : yaml.

    Returns:
        formatted_fligth_record (string)

    """
    if pretty in [1, 2]:

        flight_record_formatted = json.dumps(flight_record, indent=4)

        if pretty == 2:

            flight_record_formatted = highlight(
                    flight_record_formatted,
                    lexers.JsonLexer(),
                    formatters.TerminalFormatter())

    elif pretty == 3:

        flight_record_formatted = yaml.safe_dump(
                flight_record,
                default_flow_style=False,
                sort_keys=False)

    else:

        flight_record_formatted = json.dumps(flight_record)

    return flight_record_formatted


def _lookup(args):
    """Function to look up and process individual flight records.

    Args:
        args (argparse.Namespace)
            options:
                 --pretty {0,1,2,3}, -p {0,1,2,3}
                        option to pretty-print output.
                        0 : json unformatted (default).
                        1 : json formatted.
                        2 : json formatted & colorful.
                        3 : yaml.
            flight_numbers (string)

    Returns:
        all_flight_records (string) : flight records separated by newline.
            The value of '--pretty' determines the formatting.
    """

    FLEET["aircraft_registration"] = FLEET["Reg"]
    joined = SCHEDULE.merge(FLEET, on="aircraft_registration")

    flight_numbers = args.flight_numbers.split(",")

    all_flight_records = ""
    all_flight_records_unformatted = ""

    for flight_number in flight_numbers:

        flight_record = joined[joined.flight_number == flight_number]

        if flight_record.empty:

            error_description = (
                    "Flight number " +
                    str(flight_number) +
                    " not found.")

            flight_record = {'error': error_description}

        else:

            flight_record = flight_record.to_dict(orient="list")

            for key in ["F", "C", "E", "M",
                    "RangeLower", "RangeUpper", "Reg"]:
                del flight_record[key]

            flight_record["total_seats"] = flight_record.pop("Total")

            for key in flight_record.keys():
                flight_record[key] = str(flight_record[key][0])

            departure_airport = flight_record['departure_airport']
            arrival_airport   = flight_record['arrival_airport']

            flight_record['distance_nm'] = str(
                    _calculate_nmiles(departure_airport, arrival_airport))

        if all_flight_records != "":
            all_flight_records += '\n'
            all_flight_records_unformatted += '\n'

        flight_record_formatted = _format_flight_record(
                flight_record,
                args.pretty)

        all_flight_records             += flight_record_formatted
        all_flight_records_unformatted += json.dumps(flight_record)

    LOGGER.info(
            'SUCCESS: Lookup query with arguments %s. Response %s.',
            args.flight_numbers,
            all_flight_records_unformatted)

    return all_flight_records

def _merge(args):
    """Function to generate file 'output.csv' containing the merged
    schedule dataset.

    Args:
        args (argparse.Namespace)

    Returns:
        empty string
    """

    FLEET["aircraft_registration"] = FLEET["Reg"]

    joined = SCHEDULE.merge(FLEET, on="aircraft_registration")

    AIRPORTS["departure_airport"] = AIRPORTS["Airport"]

    joined = joined.merge(
            AIRPORTS,
            on="departure_airport",
            suffixes=(None, "_departure"))

    AIRPORTS["arrival_airport"] = AIRPORTS["Airport"]

    joined = joined.merge(
            AIRPORTS,
            on="arrival_airport",
            suffixes=(None, "_arrival"))

    joined.drop(columns=["departure_airport_arrival"], inplace=True)

    joined['distance_nm'] = joined.apply(
            lambda x:
                _calculate_nmiles(x.departure_airport, x.arrival_airport),
            axis=1)

    joined.to_csv("output.csv")

    LOGGER.info('SUCCESS Merge query')

    return ""

def query(args):
    """Function that parses the arguments of a query and calls the
    _lookup or the _merge function.

    Args:
        args (list of string): arguments to be parsed by argparse

    Returns:
        (string) Depends on whether it's 'lookup' or 'merge' mode:
        - return of _lookup:
            all_flight_records (string) : flight records separated by newline.
        - return of _merge:
            empty string

    """

    parser = argparse.ArgumentParser(description='Get schedule data.')

    subparsers = parser.add_subparsers()

    subparsers.required = True

    parser_lookup = subparsers.add_parser(
            'lookup',
            help="""Look up an individual flight records.
                Takes as argument one or multiple fligth
                numbers. If multiple, provide a list of
                flights separated by commas, eg lookup
                "ZG2361,ZG2362".""")

    parser_lookup.add_argument(
            'flight_numbers',
            help='Comma separated flight numbers.')

    parser_lookup.set_defaults(func=_lookup)

    parser_merge = subparsers.add_parser(
            'merge',
            help='Generate the merged schedule dataset in file "output.csv".')

    parser_merge.set_defaults(func=_merge)

    parser.add_argument(
            '--pretty',
            '-p',
            type=int,
            choices=[0, 1, 2, 3],
            help="""option to pretty-print output.

                    0 : json unformatted (default).

                    1 : json formatted.

                    2 : json formatted & colorful.

                    3 : yaml.""")

    ## Pops the first argument, which is the command name: 'schedule_data'
    args.pop(0)

    args_par = parser.parse_args(args)

    return args_par.func(args_par)
